unit UDFLib;

interface

uses
  IBExternals, IBHeader, IBIntf;

function BlobCompare(Blob1, Blob2: PBlob): integer; cdecl;
function Derivative(var AValue: integer): integer; cdecl;
function Hour(var ib_time: TISC_TIMESTAMP): Integer; cdecl;
function Left(sz: Pchar; var Len: integer; szRes: PChar): PChar; cdecl;
function Modulo(var i, j: Integer): Integer; cdecl;
function Year(var ib_date: TISC_TIMESTAMP): Integer; cdecl;

exports
  BlobCompare, Derivative, Hour, Left, Modulo, Year;

implementation

uses
  Math;

function BlobCompare(Blob1, Blob2: PBlob): integer; cdecl;
var
  abBuffer1, abBuffer2: string;
  bBlob1Assigned, bBlob2Assigned: boolean;
  iBlob1Byte, iBlob1BytesRead, iBlob1Segment,
  iBlob2Byte, iBlob2BytesRead, iBlob2Segment: Integer;
begin
  // only compare if both Blobs are assigned.
  bBlob1Assigned := Assigned(Blob1) and Assigned(Blob1^.BlobHandle);
  bBlob2Assigned := Assigned(Blob2) and Assigned(Blob2^.BlobHandle);
  if (bBlob1Assigned and (not bBlob2Assigned))
        or
     ((not bBlob1Assigned) and bBlob2Assigned) then begin
    // one Blob is assigned, the other isn't.  Return false.
    Result := 0;
  end else if (not bBlob1Assigned) and (not bBlob2Assigned) then begin
    // Both blobs unassigned.
    // Since NULL does not equal NULL, return false.
    // change this line if you decide the SQL standard isn't your cup of tea.
    Result := 0;
  end else begin
    // compare lengths
    if Blob1^.TotalSize <> Blob2^.TotalSize then begin
      Result := 0;
      Exit;
    end; // if
    // presume success
    Result := 1;
    // allocate buffers
    SetLength(abBuffer1, Blob1^.MaxSegmentLength * SizeOf(Byte));
    SetLength(abBuffer2, Blob2^.MaxSegmentLength * SizeOf(Byte));
    Blob2^.GetSegment(Blob2^.BlobHandle, PChar(abBuffer2), Blob2^.MaxSegmentLength, iBlob2BytesRead);
    iBlob2Segment := 1;
    iBlob2Byte := 0;
    iBlob1Segment := 0;
    repeat
      Inc(iBlob1Segment);
      Blob1^.GetSegment(Blob1^.BlobHandle, PChar(abBuffer1), Blob1^.MaxSegmentLength, iBlob1BytesRead);
      for iBlob1Byte := 0 to Pred(iBlob1BytesRead) do begin
        // this is the actual comparison of data
        if abBuffer1[iBlob1Byte] <> abBuffer2[iBlob2Byte] then begin
          // found a difference; exit
          Result := 0;
          Exit;
        end;
        Inc(iBlob2Byte);
        // see if we need to get a new segment of Blob2
        if iBlob2Byte = iBlob2BytesRead then begin
          // found the end of this segment; get the next one
          // first, make sure we're not at the end of both blobs
          if (iBlob2Segment = Blob2^.SegmentCount)
             and (iBlob1Segment = Blob1^.SegmentCount) then begin
             Exit;
          end; // if
          // more segments of Blob2 are left
          Blob2^.GetSegment(Blob2^.BlobHandle, PChar(abBuffer2), Blob2^.MaxSegmentLength, iBlob2BytesRead);
          Inc(iBlob2Segment);
          iBlob2Byte := 0;
        end; // if at end of Blob2 segment
      end; // for
    until iBlob1Segment = Blob1^.SegmentCount;
  end; // if
end;

function Derivative(var AValue: integer): integer;
begin
  Result := 0;
end;

function Hour(var ib_time: TISC_TIMESTAMP): Integer;
var
  tm_date: TM;
begin
  isc_decode_timestamp(@ib_time, @tm_date);
  result := tm_date.tm_hour;
end;

function Left(sz: Pchar; var Len: integer; szRes: PChar): PChar;
var
  i: integer;
begin
  { First, make Result point to same memory as szRes }
  result := szRes;
  { Initialize counter }
  i := 0;
  while (sz^ <> #0) and (i < Len) do begin
    { Copy one character }
    szRes^ := sz^;
    { Increment pointers to next character }
    Inc(i);
    Inc(sz);
    Inc(szRes);
  end;
  { Terminate result string }
  szRes^ := #0;
end;

function Modulo(var i, j: Integer): Integer;
begin
  // Avoid raising an EdivByZero exception
  if (j = 0) then
    result := -1 // just check the boundary condition, and
                 // return a reasonably uninteresting answer.
  else
    result := i mod j;
end;

function Year(var ib_date: TISC_TIMESTAMP): Integer;
var
  tm_date: TM;
begin
  isc_decode_timestamp(@ib_date, @tm_date);
  result := tm_date.tm_year + 1900;
end;

initialization
  IsMultiThread := TRUE;
  CheckIBLoaded;

end.
